/*
 * Control a web browser from your java application.
 * Copyright (C) 2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */
package com.Ostermiller.TulipChain;

import java.io.*;
import java.text.MessageFormat;
import java.net.URL;
import java.net.URLDecoder;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Properties;
import java.util.Vector;
import java.io.StringReader;
import java.lang.reflect.*;
import java.util.*;
import com.Ostermiller.util.Browser;

/**
 * A modal dialog that presents the configuration options.
 */
public class OptionsDialog extends JDialog {

	/**
	 * Properties that are used:
	 * max_threads
	 * proxy.host
	 * proxy.port
	 * proxy.user
	 * proxy.pass
	 * proxy.useSystem
	 */
	public void setProps(Properties props){
		this.props = props;
		if (props.containsKey("max_threads")){
			maxThreads.setText(props.getProperty("max_threads"));
		}
		if (props.containsKey("proxy.port")){
			proxyPort.setText(props.getProperty("proxy.port"));
		}
		if (props.containsKey("proxy.user")){
			proxyUser.setText(props.getProperty("proxy.user"));
		}
		if (props.containsKey("proxy.pass")){
			proxyPass.setText(props.getProperty("proxy.pass"));
		}
		if (props.containsKey("proxy.host")){
			proxyHost.setText(props.getProperty("proxy.host"));
		} else {
			proxyPort.setText("");
			proxyUser.setText("");
			proxyPass.setText("");
		}
		latestHours.setText(props.getProperty("latest_hours", "24"));

		if (props.getProperty("proxy.useSystem", "true").equals("true")){
			systemProxy.setSelected(true);
			proxyHost.setEnabled(false);
			proxyPort.setEnabled(false);
			proxyUser.setEnabled(false);
			proxyPass.setEnabled(false);
			proxyHostLabel.setEnabled(false);
			proxyPortLabel.setEnabled(false);
			proxyUserLabel.setEnabled(false);
			proxyPassLabel.setEnabled(false);

		} else {
			systemProxy.setSelected(false);
			proxyHost.setEnabled(true);
			proxyPort.setEnabled(true);
			proxyUser.setEnabled(true);
			proxyPass.setEnabled(true);
			proxyHostLabel.setEnabled(true);
			proxyPortLabel.setEnabled(true);
			proxyUserLabel.setEnabled(true);
			proxyPassLabel.setEnabled(true);
		}
		if (props.getProperty("latest_check", "true").equals("true")){
			latestCheck.setSelected(true);
			latestHours.setEnabled(true);
			latestHoursLabel.setEnabled(true);
		} else {
			latestCheck.setSelected(false);
			latestHours.setEnabled(false);
			latestHoursLabel.setEnabled(false);
		}
        
        String startLoad = props.getProperty("start_load", "last");
        if (startLoad.equals("last")){
        	lastCatStart.setSelected(true);
        	specificStartCat.setEnabled(false);
        } else if (startLoad.equals("none")){
        	noCatStart.setSelected(true);
        	specificStartCat.setEnabled(false);
        } else {
        	someCatStart.setSelected(true);
        	specificStartCat.setEnabled(true);
		}
        specificStartCat.setText(props.getProperty("start_load_cat", ""));

		redirectDeep.setSelected(props.getProperty("redirect.allowDeep", "false").equals("true"));
		redirectMetaTime.setText(props.getProperty("redirect.meta.time", "30"));
		if (props.containsKey("redirect.errors")){
			redirectDemote.setText(props.getProperty("redirect.errors").replace(';', '\n').trim());
		}
		if (props.containsKey("redirect.oks")){
			redirectPromote.setText(props.getProperty("redirect.oks").replace(';', '\n').trim());
		}
        for (int i=0; i<availableLanguages.getItemCount(); i++){
            if (((LangItem)availableLanguages.getItemAt(i)).getLang().equals(props.getProperty("locale"))){
                availableLanguages.setSelectedIndex(i);
            }
        }

		pack();
	}

	/**
	 * properties file to which to write changes.
	 */
	private Properties props;

	/**
	 * Max Threads area
	 */
	private JTextField maxThreads, latestHours, proxyHost, proxyPort, proxyUser, 
    		redirectMetaTime, specificStartCat;

	private JTextArea redirectPromote, redirectDemote;

	private JPasswordField proxyPass;

	private JCheckBox systemProxy, redirectDeep, latestCheck, useEditingAsPublicWhenPossibleCheck;

	private JComboBox editingServer, publicServer;

	private JRadioButton lastCatStart, noCatStart, someCatStart;
    
    /**
	 * The language field.
	 */
	private JComboBox availableLanguages;


	/**
	 * The label for the max threads area
	 */
	private JLabel maxThreadsLabel, proxyHostLabel, proxyPortLabel, proxyUserLabel, proxyPassLabel,
		redirectPromoteLabel, redirectDemoteLabel, latestHoursLabel, redirectMetaTimeLabel,
		editingServerLabel, publicServerLabel;

	/**
	 * The OK button.
	 */
	private JButton okButton;

	/**
	 * The cancel button.
	 */
	private JButton cancelButton;

	/**
	 * if Ok was pressed.
	 */
	private boolean pressed_OK;

	/**
	 * Create this dialog with the given parent.
	 *
	 * @param parent window from which this dialog is launched .
	 * @param props property file that is read and manipulated.
	 */
	public OptionsDialog(Frame parent, Properties props) {
		super(parent, TulipChain.labels.getString("options_dialog_title"), true);
		setProps(props);
		setLocationRelativeTo(parent);
		// super calls dialogInit, so we don't need to do it again.
	}

	/**
	 * Called by constructors to initialize the dialog.
	 */
	protected void dialogInit(){

		super.dialogInit();

		getContentPane().add(getTabs(), BorderLayout.CENTER);
		getContentPane().add(getButtons(), BorderLayout.SOUTH); 

		pack();
	}
    
    private JTabbedPane getTabs(){
    
    	JTabbedPane tabbedPane = new JTabbedPane();

		tabbedPane.addTab(
			TulipChain.labels.getString("options_dialog_connection"),
			null,
			getConnectionPane(),
			TulipChain.labels.getString("options_dialog_connection_description")
		);
		tabbedPane.setSelectedIndex(0);

		tabbedPane.addTab(
			TulipChain.labels.getString("options_dialog_redirect"),
			null,
			getRedirectPane(),
			TulipChain.labels.getString("options_dialog_redirect_description")
		);  

		tabbedPane.addTab(
			TulipChain.labels.getString("options_dialog_startup"),
			null,
			getStartupPane(),
			TulipChain.labels.getString("options_dialog_startup_description")
		);
        
		tabbedPane.addTab(
			TulipChain.labels.getString("options_dialog_browser"),
			null,
			Browser.getDialogPanel(this),
			TulipChain.labels.getString("options_dialog_browser_description")
		);
        
        return tabbedPane;
    }
    
    private JPanel getConnectionPane(){
    
    	JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
    
    	JPanel maxThreadsPane = new JPanel(new FlowLayout());
		maxThreadsLabel = new JLabel(TulipChain.labels.getString("options_dialog_max_threads") + " ");
		maxThreadsPane.add(maxThreadsLabel);
        maxThreads = new JTextField("" + ThreadManager.MAX_THREADS, 3);
		maxThreadsPane.add(maxThreads);
        panel.add(maxThreadsPane);        
        
    	panel.add(getProxyPane());
                        
    	panel.add(getDmozServerPane());
        
        return panel;
    }
    
    private JPanel getDmozServerPane(){
        
    	JPanel panel = new JPanel(new BorderLayout());
        
        panel.setBorder(createBorder(TulipChain.labels.getString("options_dialog_dmoz_server")));
        
        JPanel userConfigServerPanel = new JPanel(new SpringLayout());
        
		editingServerLabel = new JLabel(TulipChain.labels.getString("options_dialog_editing_host") + " ");
		userConfigServerPanel.add(editingServerLabel);        
        editingServer = new JComboBox(new String[]{
			"editors.dmoz.org:8080",
			"editors.dmoz.org",
			"dmoz.org:8080",
			"dmoz.org",
		});
		editingServer.setEditable(true);
		editingServer.setSelectedItem(TulipChain.getEditingHost());
		userConfigServerPanel.add(editingServer); 
        
		publicServerLabel = new JLabel(TulipChain.labels.getString("options_dialog_public_host") + " ");
		userConfigServerPanel.add(publicServerLabel);        
		publicServer = new JComboBox(new String[]{
			"dmoz.org",
			"ch.dmoz.org",
		});
		publicServer.setEditable(true);
		publicServer.setSelectedItem(TulipChain.getPublicHost());
		userConfigServerPanel.add(publicServer);        
        
        //Lay out the panel.
		makeCompactGrid(
        	userConfigServerPanel,
            2, 2, // rows, cols
            6, 6, // initX, initY
            6, 6  // xPad, yPad
        );
        
        panel.add(userConfigServerPanel, BorderLayout.CENTER);  

		useEditingAsPublicWhenPossibleCheck = new JCheckBox(TulipChain.labels.getString("options_dialog_use_editing_as_public"), true);
		panel.add(useEditingAsPublicWhenPossibleCheck, BorderLayout.SOUTH);
        
        return panel;
    }
    
    
    private Border createBorder(String name){    
        return BorderFactory.createCompoundBorder(
            BorderFactory.createTitledBorder(name),
            BorderFactory.createEmptyBorder(5,5,5,5)
        );
    }    
    
    private JPanel getProxyPane(){

    	JPanel panel = new JPanel(new BorderLayout());
        
        panel.setBorder(createBorder(TulipChain.labels.getString("options_dialog_proxy")));
        
        systemProxy = new JCheckBox(TulipChain.labels.getString("options_dialog_proxy_system"), true);
		systemProxy.addActionListener(
        	new ActionListener() {
			    public void actionPerformed(ActionEvent e){
					if (systemProxy.isSelected()){
						proxyHost.setEnabled(false);
						proxyPort.setEnabled(false);
						proxyUser.setEnabled(false);
						proxyPass.setEnabled(false);
						proxyHostLabel.setEnabled(false);
						proxyPortLabel.setEnabled(false);
						proxyUserLabel.setEnabled(false);
						proxyPassLabel.setEnabled(false);
					} else {
						proxyHost.setEnabled(true);
						proxyPort.setEnabled(true);
						proxyUser.setEnabled(true);
						proxyPass.setEnabled(true);
						proxyHostLabel.setEnabled(true);
						proxyPortLabel.setEnabled(true);
						proxyUserLabel.setEnabled(true);
						proxyPassLabel.setEnabled(true);
					}
				}
            }
        );
        panel.add(systemProxy, BorderLayout.NORTH);
        
        		
        JPanel userConfigProxyPanel = new JPanel(new SpringLayout());
 
        proxyHostLabel = new JLabel(TulipChain.labels.getString("options_dialog_proxy_host") + " ");
		proxyHostLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        userConfigProxyPanel.add(proxyHostLabel);
        proxyHost = new JTextField("", 30);
		userConfigProxyPanel.add(proxyHost);
                
        proxyPortLabel = new JLabel(TulipChain.labels.getString("options_dialog_proxy_port") + " ");
		proxyPortLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        userConfigProxyPanel.add(proxyPortLabel);
        proxyPort = new JTextField("", 5);
		userConfigProxyPanel.add(proxyPort);
        
        proxyUserLabel = new JLabel(TulipChain.labels.getString("options_dialog_proxy_user") + " ");
		proxyUserLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        userConfigProxyPanel.add(proxyUserLabel);
        proxyUser = new JTextField("", 10);
		userConfigProxyPanel.add(proxyUser);
        
        proxyPassLabel = new JLabel(TulipChain.labels.getString("options_dialog_proxy_pass") + " ");
		proxyPassLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
        userConfigProxyPanel.add(proxyPassLabel);
        proxyPass = new JPasswordField("", 10);
		userConfigProxyPanel.add(proxyPass);
        
        // Lay out the panel.
		makeCompactGrid(
        	userConfigProxyPanel,
            4, 2, // rows, cols
            6, 6, // initX, initY
            6, 6  // xPad, yPad
        );
        
        panel.add(userConfigProxyPanel, BorderLayout.CENTER);   
        
        return panel;

    }
    
    /* Used by makeCompactGrid. */
    private static SpringLayout.Constraints getConstraintsForCell(int row, int col, Container parent, int cols) {
        SpringLayout layout = (SpringLayout) parent.getLayout();
        Component c = parent.getComponent(row * cols + col);
        return layout.getConstraints(c);
    }
    
    /**
     * Aligns the first <code>rows</code> * <code>cols</code>
     * components of <code>parent</code> in
     * a grid. Each component in a column is as wide as the maximum
     * preferred width of the components in that column;
     * height is similarly determined for each row.
     * The parent is made just big enough to fit them all.
     *
     * @param rows number of rows
     * @param cols number of columns
     * @param initialX x location to start the grid at
     * @param initialY y location to start the grid at
     * @param xPad x padding between cells
     * @param yPad y padding between cells
     */
    public static void makeCompactGrid(Container parent, int rows, int cols, int initialX, int initialY, int xPad, int yPad) {
        SpringLayout layout = (SpringLayout)parent.getLayout();

        //Align all cells in each column and make them the same width.
        Spring x = Spring.constant(initialX);
        for (int c = 0; c < cols; c++) {
            Spring width = Spring.constant(0);
            for (int r = 0; r < rows; r++) {
                width = Spring.max(
                	width,
                    getConstraintsForCell(r, c, parent, cols).getWidth()
                );
            }
            for (int r = 0; r < rows; r++) {
                SpringLayout.Constraints constraints = getConstraintsForCell(r, c, parent, cols);
                constraints.setX(x);
                constraints.setWidth(width);
            }
            x = Spring.sum(x, Spring.sum(width, Spring.constant(xPad)));
        }

        //Align all cells in each row and make them the same height.
        Spring y = Spring.constant(initialY);
        for (int r = 0; r < rows; r++) {
            Spring height = Spring.constant(0);
            for (int c = 0; c < cols; c++) {
                height = Spring.max(
                	height,
                    getConstraintsForCell(r, c, parent, cols).getHeight()
                );
            }
            for (int c = 0; c < cols; c++) {
                SpringLayout.Constraints constraints = getConstraintsForCell(r, c, parent, cols);
                constraints.setY(y);
                constraints.setHeight(height);
            }
            y = Spring.sum(y, Spring.sum(height, Spring.constant(yPad)));
        }

        //Set the parent's size.
        SpringLayout.Constraints pCons = layout.getConstraints(parent);
        pCons.setConstraint(SpringLayout.SOUTH, y);
        pCons.setConstraint(SpringLayout.EAST, x);
    }
    
    private JPanel getRedirectPane(){

		redirectDeep = new JCheckBox(TulipChain.labels.getString("options_dialog_redirect_deep"), false);
		redirectPromoteLabel = new JLabel(TulipChain.labels.getString("options_dialog_redirect_promote"));
		redirectDemoteLabel = new JLabel(TulipChain.labels.getString("options_dialog_redirect_demote"));
		redirectPromote = new JTextArea("", 7, 40);
		redirectDemote = new JTextArea("", 7, 40);
		redirectMetaTime = new JTextField("", 5);
		redirectMetaTimeLabel = new JLabel(TulipChain.labels.getString("options_dialog_redirect_meta_time"));

    	GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		c.insets.top = 5;
		c.insets.bottom = 5;
		JPanel pane = new JPanel(gridbag);
		pane.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
        
    	JScrollPane scrollPane;

		c.gridy = 0;
		c.gridwidth = 1;
		c.gridx = 0;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(redirectDemoteLabel, c);
		pane.add(redirectDemoteLabel);

		c.gridy = 1;
		scrollPane = new JScrollPane(redirectDemote);
		gridbag.setConstraints(scrollPane, c);
		pane.add(scrollPane);

		c.gridy = 2;
		gridbag.setConstraints(redirectDeep, c);
		pane.add(redirectDeep);

		c.gridy = 3;
		gridbag.setConstraints(redirectPromoteLabel, c);
		pane.add(redirectPromoteLabel);

		c.gridy = 4;
		scrollPane = new JScrollPane(redirectPromote);
		gridbag.setConstraints(scrollPane, c);
		pane.add(scrollPane);

		c.gridy=5;
		gridbag.setConstraints(redirectMetaTimeLabel, c);
		pane.add(redirectMetaTimeLabel);

		c.gridy=6;
		gridbag.setConstraints(redirectMetaTime, c);
		pane.add(redirectMetaTime);
        
        return pane;
    }
    
    private JPanel getStartupPane(){    
    
    	JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
        
    	panel.add(getStartCategoryPane());
                        
    	panel.add(getUpdateCheckPane());        
                        
    	panel.add(getLanguagePane());
        
        return panel;
    }
    
    private JPanel getLanguagePane(){
    	
        availableLanguages = new JComboBox(new LangItem[]{
            new LangItem("default",TulipChain.labels.getString("lang_default")),
            new LangItem("da",TulipChain.labels.getString("lang_da")),
            new LangItem("de",TulipChain.labels.getString("lang_de")),
            new LangItem("en_GB",TulipChain.labels.getString("lang_en_GB")),
            new LangItem("en",TulipChain.labels.getString("lang_en")),
            new LangItem("en_US",TulipChain.labels.getString("lang_en_US")),
            new LangItem("es",TulipChain.labels.getString("lang_es")),
            new LangItem("fr",TulipChain.labels.getString("lang_fr")),
            new LangItem("it",TulipChain.labels.getString("lang_it")),
            new LangItem("pl",TulipChain.labels.getString("lang_pl")),
        });
          
    	JPanel panel = new JPanel(new BorderLayout());        
        panel.setBorder(createBorder(TulipChain.labels.getString("options_dialog_language")));
        panel.add(availableLanguages);
        return panel;
    }   
        
    private JPanel getStartCategoryPane(){    
    	JPanel panel = new JPanel(new SpringLayout());        
        panel.setBorder(createBorder(TulipChain.labels.getString("options_dialog_start_cat")));
        
        ButtonGroup startCatGroup = new ButtonGroup();
		ActionListener startCatActionListener = new ActionListener(){
			public void actionPerformed(ActionEvent e){
            	specificStartCat.setEnabled(e.getSource() == someCatStart);
			}
		};
        
		lastCatStart = new JRadioButton(TulipChain.labels.getString("options_dialog_last_cat_start"));
		panel.add(lastCatStart);
        startCatGroup.add(lastCatStart);
        lastCatStart.addActionListener(startCatActionListener);
        panel.add(new JPanel());
        noCatStart = new JRadioButton(TulipChain.labels.getString("options_dialog_no_cat_start"));
		panel.add(noCatStart);
        startCatGroup.add(noCatStart);
        noCatStart.addActionListener(startCatActionListener);
        panel.add(new JPanel());
        someCatStart = new JRadioButton(TulipChain.labels.getString("options_dialog_some_cat_start"));
		panel.add(someCatStart);
        startCatGroup.add(someCatStart);
        someCatStart.addActionListener(startCatActionListener);
        specificStartCat = new JTextField("", 30);
        panel.add(specificStartCat);


        
        // Lay out the panel.
		makeCompactGrid(
        	panel,
            3, 2, // rows, cols
            6, 6, // initX, initY
            6, 6  // xPad, yPad
        );

        
        return panel;
	}
    
    private JPanel getUpdateCheckPane(){
      
    	JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
        panel.setBorder(createBorder(TulipChain.labels.getString("options_dialog_update_check")));
        
        latestCheck = new JCheckBox(TulipChain.labels.getString("options_dialog_latest"), true);
		latestCheck.addActionListener(
        	new ActionListener() {
				public void actionPerformed(ActionEvent e){
					if (latestCheck.isSelected()){
						latestHours.setEnabled(true);
						latestHoursLabel.setEnabled(true);
					} else {
						latestHours.setEnabled(false);
						latestHoursLabel.setEnabled(false);
					}
			    }
            }
        );
        panel.add(latestCheck);
        
        JPanel latestHoursPanel = new JPanel(new FlowLayout());
		latestHoursLabel = new JLabel(TulipChain.labels.getString("options_dialog_latest_hours") + " ");
        latestHoursPanel.add(latestHoursLabel);
        latestHours = new JTextField("", 3);
		latestHoursPanel.add(latestHours);
        
        panel.add(latestHoursPanel);
        
        return panel; 
    }
    
    private JPanel getButtons(){
    	    
    	JPanel panel = new JPanel(new FlowLayout());
        
		okButton = new JButton(TulipChain.labels.getString("options_dialog_ok"));        
        okButton.addActionListener(
        	new ActionListener() {
			    public void actionPerformed(ActionEvent e){
				    pressed_OK = true;
				    OptionsDialog.this.hide();
                }
            }
        );        
		panel.add(okButton);
        
        cancelButton = new JButton(TulipChain.labels.getString("options_dialog_cancel"));
        cancelButton.addActionListener(
        	new ActionListener() {
			    public void actionPerformed(ActionEvent e){
				    pressed_OK = false;
				    OptionsDialog.this.hide();
                }
            }
        );        
		panel.add(cancelButton);
        
        return panel;    	
    }

	/**
	 * Shows the dialog.
	 */
	public void show(){
		pressed_OK = false;
        Browser.initPanel();
		super.show();
		if (pressed_OK){
        	Browser.userOKedPanelChanges();
			try {
				ThreadManager.MAX_THREADS = Integer.parseInt(maxThreads.getText().trim(), 10);
				if (ThreadManager.MAX_THREADS != 20 || props.getProperty("max_threads")==null){
					props.put("max_threads", "" + ThreadManager.MAX_THREADS);
				} else {
					props.remove("max_threads");
				}
			} catch (NumberFormatException x){
			}           
            
            String startLoad = props.getProperty("start_load", "last");
            if (lastCatStart.isSelected()){
        	    props.setProperty("start_load", "last");
		    } else if (noCatStart.isSelected()){
        	    props.setProperty("start_load", "none");	
            } else {
        	    props.setProperty("start_load", "some");
            }
            props.setProperty("start_load_cat", specificStartCat.getText());
            
			props.setProperty("editing.host", (String)(editingServer.getSelectedItem()));
			props.setProperty("public.host", (String)(publicServer.getSelectedItem()));
			props.setProperty("public.pages.from.editing.host", Boolean.toString(useEditingAsPublicWhenPossibleCheck.isSelected()));
			props.put("proxy.useSystem", "" + systemProxy.isSelected());
			if (!systemProxy.isSelected()){
				if (proxyHost.getText().trim().equals("")){
					props.remove("proxy.host");
					props.remove("proxy.port");
					props.remove("proxy.user");
					props.remove("proxy.pass");
				} else {
					props.put("proxy.host", proxyHost.getText().trim());
					if (proxyPort.getText().trim().equals("")){
						props.remove("proxy.port");
					} else {
						props.put("proxy.port", proxyPort.getText().trim());
					}
					if (proxyUser.getText().trim().equals("")){
						props.remove("proxy.user");
					} else {
						props.put("proxy.user", proxyUser.getText().trim());
					}
					if (new String(proxyPass.getPassword()).trim().equals("")){
						props.remove("proxy.pass");
					} else {
						props.put("proxy.pass", new String(proxyPass.getPassword()).trim());
					}
				}
			}
			TulipChain.updateRedirectErrors(
				props,
				"redirect.errors",
				props.getProperty("redirect.errors", "") + ";" +
				props.getProperty("redirect.errors.old", ""),
				redirectDemote.getText(),
				""
			);
			TulipChain.updateRedirectErrors(
				props,
				"redirect.oks",
				props.getProperty("redirect.oks", "") + ";" +
				props.getProperty("redirect.oks.old", ""),
				redirectPromote.getText(),
				""
			);
			props.put("redirect.allowDeep", "" + redirectDeep.isSelected());
			try {
				int time = Integer.parseInt(redirectMetaTime.getText().trim());
				props.put("redirect.meta.time", Integer.toString(time));
			} catch (NumberFormatException nfe){
			}
			props.put("latest_hours", latestHours.getText());
			props.put("latest_check", "" + latestCheck.isSelected());
			props.put("properties.updated", "" + System.currentTimeMillis());
            props.put("locale",((LangItem)availableLanguages.getSelectedItem()).getLang());
		}
	}    
    
    private class LangItem implements Comparable {
        private String lang;
        private String text;        
        public LangItem(String lang, String text){
            this.lang=lang;
            this.text=text;
        }
        public String getLang(){
            return lang;
        }
        public String toString(){
            return text;
        }
        public int compareTo(Object o){
            if (!(o instanceof LangItem)) return -1;
            return (this.lang.compareTo(((LangItem)o).lang));
        }
    }
}
