/*
 * Control a web browser from your java application.
 * Copyright (C) 2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */
package com.Ostermiller.TulipChain;

import java.io.*;
import java.text.MessageFormat;
import java.net.URL;
import java.net.URLDecoder;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Properties;
import java.util.Vector;
import java.util.StringTokenizer;

/**
 * A modal dialog that presents the configuration options.
 */
public class ReportDialog extends JDialog {

	private static final int FILE_HISTORY_COUNT = 10;

	private Object selectedReport = null;

	/**
	 * Properties that are used:
	 * report.unchecked
	 * report.ok
	 * report.slash
	 * report.warning
	 * report.connectionError
	 * report.error
	 * report.file
	 * report.newWindows
	 * report.autoCorrect
	 * report.showEmptyCats
	 * report.includeReviewed
	 * report.includeUnreviewed
	 */
	public void setProps(Properties props){
		this.props = props;

		unchecked.setSelected(props.getProperty("report.unchecked", "false").equals("true"));
		ok.setSelected(props.getProperty("report.ok", "false").equals("true"));
		slash.setSelected(props.getProperty("report.slash", "true").equals("true"));
		warning.setSelected(props.getProperty("report.warning", "true").equals("true"));
		connectionError.setSelected(props.getProperty("report.connectionError", "true").equals("true"));
		error.setSelected(props.getProperty("report.error", "true").equals("true"));
		newWindows.setSelected(props.getProperty("report.newWindows", "false").equals("true"));
		ordered.setSelected(props.getProperty("report.ordered", "false").equals("true"));
		autoCorrect.setSelected(props.getProperty("report.autoCorrect", "false").equals("true"));
		showEmptyCats.setSelected(props.getProperty("report.showEmptyCats", "true").equals("true"));
		includeReviewed.setSelected(props.getProperty("report.includeReviewed", "true").equals("true"));
		includeUnreviewed.setSelected(props.getProperty("report.includeUnreviewed", "true").equals("true"));

		fileField.removeAllItems();
		boolean set = false;
		if (props.containsKey("report.file")){
			StringTokenizer tok = new StringTokenizer(
				props.getProperty("report.file", ""),
				System.getProperty("path.separator"),
				false
			);
			for (int i=0; tok.hasMoreTokens(); i++){
				String path = tok.nextToken();
				fileField.addItem(path);
				if (i==0){
					selectedReport = path;
				}
				set = true;
			}
		}
		if (!set){
			File reportFile = new File(System.getProperty("user.home"), ".java");
			reportFile = new File(reportFile, "TulipChain");
			reportFile = new File(reportFile, "Report.html");
			String path = reportFile.getPath();
			fileField.addItem(path);
			selectedReport = path;
		}
        for (int i=0; i<availableLanguages.getItemCount(); i++){
            if (((LangItem)availableLanguages.getItemAt(i)).getLang().equals(props.getProperty("report.language"))){
                availableLanguages.setSelectedIndex(i);
            }
        }
	}

	/**
	 * properties file to which to write changes.
	 */
	private Properties props;

	private JCheckBox unchecked, ok, slash, warning, connectionError, error, newWindows,
		ordered, autoCorrect, showEmptyCats, includeReviewed, includeUnreviewed;

	/**
	 * The OK button.
	 */
	private JButton okButton;

	/**
	 * The browse button.
	 */
	private JButton browseButton;

	/**
	 * The File Field.
	 */
	private JComboBox fileField;
    
    /**
	 * The language field.
	 */
	private JComboBox availableLanguages;

	 /**
	 * The File Label.
	 */
	private JLabel fileLabel;
    
    /** 
     * The Language Label
     */
    private JLabel languageLabel;

	/**
	 * The cancel button.
	 */
	private JButton cancelButton;

	/**
	 * if Ok was pressed.
	 */
	private boolean pressed_OK;

	/**
	 * the file chooser.
	 */
	private JFileChooser chooser;

	/**
	 * Create this dialog with the given parent.
	 *
	 * @param parent window from which this dialog is launched .
	 * @param props property file that is read and manipulated.
	 */
	public ReportDialog(Frame parent, Properties props) {
		super(parent, "TulipChain Options", true);
		setProps(props);
		setLocationRelativeTo(parent);
		// super calls dialogInit, so we don't need to do it again.
	}

	/**
	 * Called by constructors to initialize the dialog.
	 */
	protected void dialogInit(){

		unchecked = new JCheckBox();
		ok = new JCheckBox();
		slash = new JCheckBox();
		warning = new JCheckBox();
		connectionError = new JCheckBox();
		error = new JCheckBox();
		newWindows = new JCheckBox();
		ordered = new JCheckBox();
		autoCorrect = new JCheckBox();
		showEmptyCats = new JCheckBox();
		includeReviewed = new JCheckBox();
		includeUnreviewed = new JCheckBox();

		okButton = new JButton();
		cancelButton = new JButton();
		browseButton = new JButton();

		fileField = new JComboBox();
		fileField.setEditable(true);
        
        availableLanguages = new JComboBox(new LangItem[]{
            new LangItem("default",TulipChain.labels.getString("lang_default")),
            new LangItem("da",TulipChain.labels.getString("lang_da")),
            new LangItem("de",TulipChain.labels.getString("lang_de")),
            new LangItem("en_GB",TulipChain.labels.getString("lang_en_GB")),
            new LangItem("en",TulipChain.labels.getString("lang_en")),
            new LangItem("en_US",TulipChain.labels.getString("lang_en_US")),
            new LangItem("es",TulipChain.labels.getString("lang_es")),
            new LangItem("fr",TulipChain.labels.getString("lang_fr")),
            new LangItem("it",TulipChain.labels.getString("lang_it")),
            new LangItem("pl",TulipChain.labels.getString("lang_pl")),
        });
        
		fileLabel = new JLabel();
        languageLabel = new JLabel();
        
		super.dialogInit();

		setTitle(TulipChain.labels.getString("report_dialog_title"));
		okButton.setText(TulipChain.labels.getString("report_dialog_ok"));
		cancelButton.setText(TulipChain.labels.getString("report_dialog_cancel"));
		unchecked.setText(TulipChain.labels.getString("report_dialog_unchecked"));
		ok.setText(TulipChain.labels.getString("report_dialog_oksites"));
		slash.setText(TulipChain.labels.getString("report_dialog_slash"));
		warning.setText(TulipChain.labels.getString("report_dialog_warning"));
		connectionError.setText(TulipChain.labels.getString("report_dialog_connection_error"));
		error.setText(TulipChain.labels.getString("report_dialog_error"));
		newWindows.setText(TulipChain.labels.getString("report_dialog_newwindows"));
		ordered.setText(TulipChain.labels.getString("report_dialog_ordered"));
		autoCorrect.setText(TulipChain.labels.getString("report_dialog_auto_correct"));
		showEmptyCats.setText(TulipChain.labels.getString("report_dialog_show_empty_cats"));
		includeReviewed.setText(TulipChain.labels.getString("report_dialog_include_reviewed"));
		includeUnreviewed.setText(TulipChain.labels.getString("report_dialog_include_unreviewed"));
		fileLabel.setText(TulipChain.labels.getString("report_dialog_report_file"));
        languageLabel.setText(TulipChain.labels.getString("report_dialog_lang"));
		browseButton.setText(TulipChain.labels.getString("report_dialog_browse"));
        
		ActionListener actionListener = new ActionListener() {
			public void actionPerformed(ActionEvent e){
				Object source = e.getSource();
				if (source == browseButton){
					if (chooser == null){
						chooser = new JFileChooser();
					}
					chooser.setCurrentDirectory((new File((String)fileField.getSelectedItem())).getParentFile());
					int returnVal = chooser.showOpenDialog(ReportDialog.this);
					if(returnVal == JFileChooser.APPROVE_OPTION) {
						String path = chooser.getSelectedFile().getPath();
						boolean found = false;
						for (int i=0; i<fileField.getItemCount(); i++){
							if (path.equals((String)fileField.getItemAt(i))){
								found = true;
								fileField.setSelectedIndex(i);
							}
						}
						if (!found){
							fileField.addItem(path);
							fileField.setSelectedItem(path);
						}
					}
				} else {
					pressed_OK = (source == okButton);
					ReportDialog.this.hide();
				}
			}
		};

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c;
		JTabbedPane tabPane = new JTabbedPane();
		tabPane.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
		JPanel pane;
		JLabel label;

		pane = new JPanel(gridbag);
		pane.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
		c = new GridBagConstraints();
		c.insets.top = 5;
		c.insets.bottom = 5;
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 0;
		c.gridy = 0;
		gridbag.setConstraints(unchecked, c);
		pane.add(unchecked);
		c.gridy = 1;
		gridbag.setConstraints(ok, c);
		pane.add(ok);
		c.gridy = 2;
		gridbag.setConstraints(slash, c);
		pane.add(slash);
		c.gridy = 3;
		gridbag.setConstraints(warning, c);
		pane.add(warning);
		c.gridy = 4;
		gridbag.setConstraints(connectionError, c);
		pane.add(connectionError);
		c.gridy = 5;
		gridbag.setConstraints(error, c);
		pane.add(error);
		c.gridy = 6;
		gridbag.setConstraints(includeReviewed, c);
		pane.add(includeReviewed);
		c.gridy = 7;
		gridbag.setConstraints(includeUnreviewed, c);
		pane.add(includeUnreviewed);
		tabPane.addTab(TulipChain.labels.getString("report_tab_include"), pane);

		pane = new JPanel(gridbag);
		pane.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
		c = new GridBagConstraints();
		c.insets.top = 5;
		c.insets.bottom = 5;
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 0;
		c.gridy = 0;
		gridbag.setConstraints(newWindows, c);
		pane.add(newWindows);
		c.gridy = 1;
		gridbag.setConstraints(ordered, c);
		pane.add(ordered);
		c.gridy = 2;
		gridbag.setConstraints(autoCorrect, c);
		pane.add(autoCorrect);
		c.gridy = 3;
		gridbag.setConstraints(showEmptyCats, c);
		pane.add(showEmptyCats);
		tabPane.addTab(TulipChain.labels.getString("report_tab_options"), pane);
		getContentPane().add(tabPane, BorderLayout.CENTER);

		pane = new JPanel(gridbag);
		pane.setBorder(BorderFactory.createEmptyBorder(10, 20, 5, 20));
		c = new GridBagConstraints();
		c.insets.top = 5;
		c.insets.bottom = 5;
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 0;
		c.gridy = 0;
		gridbag.setConstraints(fileLabel, c);
		pane.add(fileLabel);
		c.gridy = 1;
		gridbag.setConstraints(fileField, c);
		pane.add(fileField);
		c.gridx = 1;
		gridbag.setConstraints(browseButton, c);
		browseButton.addActionListener(actionListener);
		pane.add(browseButton);        
		c.gridx = 0;
        c.gridy = 2;
        gridbag.setConstraints(languageLabel, c);
		pane.add(languageLabel);        
        c.gridy = 3;
		gridbag.setConstraints(availableLanguages, c);
		pane.add(availableLanguages);        
		tabPane.addTab(TulipChain.labels.getString("report_tab_file"), pane);

		JPanel panel = new JPanel();
		okButton.addActionListener(actionListener);
		panel.add(okButton);
		cancelButton.addActionListener(actionListener);
		panel.add(cancelButton);
		getContentPane().add(panel, BorderLayout.SOUTH);

		Dimension d = fileField.getPreferredSize();
		d.width=250;
		fileField.setPreferredSize(d);
        
        d = availableLanguages.getPreferredSize();
		d.width=250;
		availableLanguages.setPreferredSize(d);

		pack();
	}

	/**
	 * Shows the dialog.
	 */
	public void show(){
		pressed_OK = false;
		fileField.setSelectedItem(selectedReport);
		super.show();
		if (pressed_OK){
			props.put("report.unchecked", "" + unchecked.isSelected());
			props.put("report.ok", "" + ok.isSelected());
			props.put("report.slash", "" + slash.isSelected());
			props.put("report.warning", "" + warning.isSelected());
			props.put("report.connectionError", "" + connectionError.isSelected());
			props.put("report.error", "" + error.isSelected());
			String selected = (String)fileField.getSelectedItem();
			StringBuffer sb = new StringBuffer();
			if (selected != null && selected.length()!=0){
				sb.append(selected);
				sb.append(System.getProperty("path.separator"));
			}
			for(int i=0; i<FILE_HISTORY_COUNT && i<fileField.getItemCount(); i++){
				String path = (String)fileField.getItemAt(i);
				if (path != null && !selected.equals(path) && path.length() != 0){
					sb.append(path);
					sb.append(System.getProperty("path.separator"));
				}
			}
			props.put("report.file", sb.toString());
			props.put("report.newWindows", "" + newWindows.isSelected());
			props.put("report.ordered", "" + ordered.isSelected());
			props.put("report.autoCorrect", "" + autoCorrect.isSelected());
			props.put("report.showEmptyCats", "" + showEmptyCats.isSelected());
			props.put("report.includeReviewed", "" + includeReviewed.isSelected());
			props.put("report.includeUnreviewed", "" + includeUnreviewed.isSelected());
            props.put("report.language",((LangItem)availableLanguages.getSelectedItem()).getLang());

		}
	}

	public boolean ok(){
		return pressed_OK;
	}
    
    private class LangItem implements Comparable {
        private String lang;
        private String text;        
        public LangItem(String lang, String text){
            this.lang=lang;
            this.text=text;
        }
        public String getLang(){
            return lang;
        }
        public String toString(){
            return text;
        }
        public int compareTo(Object o){
            if (!(o instanceof LangItem)) return -1;
            return (this.lang.compareTo(((LangItem)o).lang));
        }
    }
}
