/*
 * TulipChain is the dmoz browser.  http://ostermiller.org/tulipchain/
 * Copyright (C) 2001-2002 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=TulipChain
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See COPYING.TXT for details.
 */

package com.Ostermiller.TulipChain;

import java.util.*;
import java.text.MessageFormat;

/**
 * Anything that Tulip Chain does that would take long enough that
 * the user would notice if it were done in the same thread as the GUI
 * should be handled by the thread manager.  The thread manager maintains
 * methods for queuing command tulipchain tasks and limits the number of
 * threads that tulipchain has going at once.
 */
public class ThreadManager extends Thread {

	/**
	 * set this to true to make this thread stop itself.
	 */
	public boolean done;

	/**
	 * The maximum number of threads this class will allow
	 * at once.  Requests beyond this will be queued.
	 */
	public static volatile int MAX_THREADS = 20;

	private Vector profilesToLoad = new Vector();
	private Vector updateChecks = new Vector();
	private Vector catsToLoad = new Vector();
	private Vector sitesToCheck = new Vector();
	private Vector catsToSelect = new Vector();
	private Vector catsToSpider = new Vector();
	private Vector threads = new Vector();
	private Vector sitesToDelete = new Vector();
	private Vector sitesToCorrect = new Vector();
	private TulipChain tulip;
	private boolean pause = false;
	private HashMap catTasks = new HashMap();

	/**
	 * The thread manager needs to know about tulip
	 * chain so that it has somewhere to report its
	 * status.
	 */
	public void setManager(TulipChain tulip){
		this.tulip = tulip;
	}

	/**
	 * Queue a thread that will check the given
	 * listing.
	 * (Lowest Priority)
	 */
	public void checkSite(DmozListing url){
		url.getURLwStatus().setProperties(tulip.props);
		sitesToCheck.add(url.getURLwStatus());
	}

	public void delete(DmozDeleter toDelete){
		sitesToDelete.add(toDelete);
	}

	public void autoCorrect(AutoCorrector ac){
		sitesToCorrect.add(ac);
	}

	/**
	 * Queue a thread that will check the given
	 * listing.
	 * (Lowest Priority)
	 */
	public void checkSite(URLStatus url){
		url.setProperties(tulip.props);
		sitesToCheck.add(url);
	}

	/**
	 * Queue a thread that will load the given
	 * category.
	 * (High Priority)
	 */
	public void loadCat(DmozCat cat){
		if (!catsToLoad.contains(cat) &&
			(cat.getThread() == null || !cat.getThread().isAlive())){
			catsToLoad.add(cat);
		}
	}

	public void checkForUpdates(boolean alertOnFail, boolean alertAlreadySeen){
		updateChecks.add(new UpdateCheck(alertOnFail, alertAlreadySeen));
	}

	public void loadProfile(){
		profilesToLoad.add(new ProfileLoader());
	}

	/**
	 * Queue a thread that will load the given
	 * category.
	 * (High Priority)
	 */
	public void spiderCat(DmozCat cat){
		catsToSpider.add(cat);
	}

	/**
	 * Queue a thread that will load the category
	 * and all parents if they are not already
	 * loaded and display the category in TulipChain.
	 * (Highest Priority)
	 */
	public void selectCat(String cat){
		catsToSelect.add(new CatSelecter(cat));
	}

	public void halt(){
		pause = true;
		for (int i=0; i<threads.size(); i++){
			Haltable t = (Haltable)threads.elementAt(i);
			t.halt();
		}
		while (profilesToLoad.size() > 0){
			profilesToLoad.remove(0);
		}
		while (updateChecks.size() > 0){
			updateChecks.remove(0);
		}
		while (catsToLoad.size() > 0){
			catsToLoad.remove(0);
		}
		while (sitesToCheck.size() > 0){
			sitesToCheck.remove(0);
		}
		while (catsToSelect.size() > 0){
			catsToSelect.remove(0);
		}
		while (catsToSpider.size() > 0){
			catsToSpider.remove(0);
		}
		while(sitesToCorrect.size() > 0){
		   sitesToCorrect.remove(0);
		}
		while (sitesToDelete.size() > 0){
			sitesToDelete.remove(0);
		}
		try {
			sleep(100);
		} catch (InterruptedException e) {}
		pause = false;
	}

	/**
	 * Should be called from the start() method to run this as a thread.
	 */
	public void run() {
		int numQueued;
		int numThreads;
		int lastNumQueued = 0;
		int lastNumThreads = 0;
		long lastDmozRequest = 0;

		while (!done){
			if (!pause){
				int max = MAX_THREADS;
				numQueued = updateChecks.size() + sitesToCheck.size() + catsToLoad.size() +
						catsToSelect.size() + sitesToDelete.size() + catsToSpider.size() +
						sitesToCorrect.size() + profilesToLoad.size();
				numThreads = threads.size();
				// if the stuff we are working on has changed since last time,
				// set the status in tulip chain to indicate this.
				if (numQueued != lastNumQueued || numThreads != lastNumThreads){
					if (numQueued == 0 && numThreads == 0){
						TulipChain.setStatus(TulipChain.labels.getString("done_status"));
					} else {
						TulipChain.setStatus(((numQueued != 0)?(
							MessageFormat.format(
								TulipChain.labels.getString("queued_status"),
								new Object[] {
									new Integer(numQueued)
								}
							) + " "):"") + MessageFormat.format(
								TulipChain.labels.getString("working_status"),
								new Object[] {
									new Integer(numThreads)
								}
							)
						);
					}
				}
				lastNumQueued = numQueued;
				lastNumThreads = numThreads;
				// see if any threads have finished.
				for (int i=0; i<threads.size(); i++){
					Thread t = ((Haltable)threads.elementAt(i)).getThread();
					if (t == null || !t.isAlive()){
						threads.remove(i);
					}
				}
				// start new threads if there is room to do so.
				// these blocks should be in order of priority.
				if (profilesToLoad.size() > 0 && threads.size() < max){
					ProfileLoader pl = (ProfileLoader)profilesToLoad.remove(0);
					pl.start();
					threads.add(pl);
				}
				if (catsToSelect.size() > 0 && threads.size() < max){
					CatSelecter cat = (CatSelecter)catsToSelect.remove(0);
					cat.start();
					threads.add(cat);
				}
				if (updateChecks.size() > 0 && threads.size() < max){
					UpdateCheck check = (UpdateCheck)updateChecks.remove(0);
					check.start();
					threads.add(check);
				}
				if (System.currentTimeMillis() > lastDmozRequest + 500){
					if (catsToLoad.size() > 0 && threads.size() < max){
						DmozCat cat = (DmozCat)catsToLoad.remove(0);
						cat.start();
						threads.add(cat);
					} else if  (catsToSpider.size() > 0 && threads.size() < max){
						DmozCat cat = (DmozCat)catsToSpider.remove(0);
						cat.setSpider(true);
						cat.start();
						threads.add(cat);
					} else {
						boolean editAction = false;
						if (!editAction && sitesToDelete.size() > 0 && threads.size() < max){
							DmozDeleter toDelete = (DmozDeleter)sitesToDelete.remove(0);
							String cat = toDelete.getCatName();
							DmozEditTask task = (DmozEditTask)catTasks.get(cat);
							if (task == null || task.getThread() == null || !task.getThread().isAlive()){
								toDelete.start();
								catTasks.put(cat, toDelete);
								threads.add(toDelete);
								editAction = true;
							} else {
								sitesToDelete.add(toDelete);
							}
						}
						if (!editAction && sitesToCorrect.size() > 0 && threads.size() < max){
							AutoCorrector ac = (AutoCorrector)sitesToCorrect.remove(0);
							String cat = ac.getCatName();
							DmozEditTask task = (DmozEditTask)catTasks.get(cat);
							if (task == null || task.getThread() == null || !task.getThread().isAlive()){
								ac.start();
								catTasks.put(cat, ac);
								threads.add(ac);
								editAction = true;
							} else {
								sitesToCorrect.add(ac);
							}
						}
					}
					lastDmozRequest = System.currentTimeMillis();
				}
				if (sitesToCheck.size() > 0 && threads.size() < max){
					URLStatus url = (URLStatus)sitesToCheck.remove(0);
					url.start();
					threads.add(url);
				}
			}
			try {
				sleep(10);
			} catch (InterruptedException e) {}
		}
	}
}
